@@ -28,7 +28,6 @@ import re
 
 import colorama
 
-
 class NullIndenter(object):
     def __enter__(self):
         return self
@@ -66,7 +65,7 @@ def _colorWrapper(m):
 def colorize(text):
     return _REGEX_COLOR.sub(_colorWrapper,text)
 
-class IndentLogger(object):
+class IndentLogger:
     '''
     Indents stuff.
     '''
