@@ -38,7 +38,7 @@ from functools import wraps
 
 # Import third party libs
 import six
-from jinja2 import Markup, nodes
+from jinja2 import nodes
 from jinja2.environment import TemplateModule
 from jinja2.ext import Extension
 from jinja2.exceptions import TemplateRuntimeError
@@ -47,6 +47,17 @@ from ruamel.yaml import YAML
 from ruamel.yaml.compat import StringIO
 from ruamel.yaml.representer import RoundTripRepresenter
 
+try:
+    from markupsafe import Markup
+except ImportError:
+    # jinja < 3.1
+    from jinja2 import Markup
+
+try:
+    contextfunction = jinja2.contextfunction
+except AttributeError:
+    contextfunction = jinja2.pass_context
+
 yaml = YAML(typ='safe', pure=True)
 
 from collections import OrderedDict
@@ -158,7 +169,7 @@ def ensure_sequence_filter(data):
     return data
 
 
-@jinja2.contextfunction
+@contextfunction
 def show_full_context(ctx):
     return ctx
 
