@@ -20,32 +20,36 @@ AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE.
-
 '''
+from __future__ import annotations
+
 import hashlib
-from inspect import trace
 import os
-from traceback import StackSummary
 import traceback
 import typing
+from inspect import trace
 from pathlib import Path
+from traceback import StackSummary
 from typing import Any, Callable, Dict, List, Optional, Set, Union
 
 from ruamel.yaml import YAML
 from ruamel.yaml.compat import StringIO
 
-yaml = YAML(typ='safe', pure=True)
-
 from buildtools import os_utils, utils
 from buildtools.bt_logging import IndentLogger, log
 from buildtools.maestro.utils import callLambda
 
+yaml = YAML(typ='safe', pure=True)
+
+
 if typing.TYPE_CHECKING:
     from . import BuildMaestro
 
+
 class BuildFailedError(Exception):
     pass
 
+
 class BuildTarget(object):
     BT_TYPE: str = '-'
     BT_COLOR: str = 'cyan'
@@ -57,11 +61,15 @@ class BuildTarget(object):
     CHECK_MTIMES: bool = True
     CHECK_HASHES: bool = True
 
-    def __init__(self, targets: Union[List[str], str]=[], files: List[str]=[], dependencies: List[str]=[], provides: List[str]=[], name: str='') -> None:
+    def __init__(self, targets: Union[List[str], str] = [],
+                 files: List[str] = [],
+                 dependencies: List[str] = [],
+                 provides: List[str] = [],
+                 name: str = '') -> None:
         self.id: int = 0
-        self.instantiated_at: StackSummary = traceback.extract_stack() # Instantiation
+        self.instantiated_at: StackSummary = traceback.extract_stack()  # Instantiation
         self.added_at: Optional[StackSummary] = None
-        self._all_provides: List[str] = targets if isinstance(targets, list) else [targets]+provides
+        self._all_provides: List[str] = targets if isinstance(targets, list) else [targets] + provides
         self.name: str = ''
         try:
             self.name = os.path.relpath(self._all_provides[0], os.getcwd()) if name == '' else name
@@ -70,7 +78,7 @@ class BuildTarget(object):
         self.files: List[str] = files
         self.dependencies: List[str] = dependencies
 
-        self.maestro: Optional['BuildMaestro'] = None
+        self.maestro: Optional[BuildMaestro] = None
 
         #: Print executed commands to console.
         self.show_commands: bool = False
@@ -78,14 +86,14 @@ class BuildTarget(object):
         #: This target was rebuilt. (Files changed)
         self.dirty: bool = False
 
-        self._lambdas_called: bool=False
+        self._lambdas_called: bool = False
 
         # Cache stuff
         self.lastConfigHash: str = ''
         self.lastTargetHash: str = ''
-        self.lastFileTimes: Dict[str, float]={}
-        self.lastFileHashes: Dict[str, str]={}
-        self.lastConfig: Dict[str, Any]={}
+        self.lastFileTimes: Dict[str, float] = {}
+        self.lastFileHashes: Dict[str, str] = {}
+        self.lastConfig: Dict[str, Any] = {}
 
     def try_build(self) -> None:
         self.files = callLambda(self.files)
@@ -100,9 +108,14 @@ class BuildTarget(object):
             for filename in self.provides():
                 self.removeFile(filename)
 
-    def removeFile(self, filename) -> None:
+    def removeFile(self, filename: str) -> None:
+        assert self.maestro is not None
         if os.path.isfile(filename):
-            log.info('<red>RM    </red> %s',filename)
+            padded_label = 'RM'.ljust(self.maestro.get_max_label_length())
+            if self.maestro.colors:
+                log.info(f'<red>{padded_label}</red> {filename}')
+            else:
+                log.info(f'{padded_label} {filename}')
             os.remove(filename)
 
     def _set_failed(self) -> None:
@@ -117,6 +130,7 @@ class BuildTarget(object):
         return {}
 
     def should_echo_commands(self) -> bool:
+        assert self.maestro is not None
         return self.maestro.show_commands or self.show_commands
 
     def is_stale(self) -> bool:
@@ -130,7 +144,7 @@ class BuildTarget(object):
         if self.getConfigHash() != self.lastConfigHash:
             log.debug('[is stale] Config hash changed')
             return True
-        if self.haveFilesChanged(): #self.checkMTimes(self.files+self.dependencies, self.provides(), config=self.get_config())
+        if self.haveFilesChanged():  # self.checkMTimes(self.files+self.dependencies, self.provides(), config=self.get_config())
             return True
         return False
 
@@ -147,12 +161,14 @@ class BuildTarget(object):
         return self.name
 
     def verboseLogEntry(self, color) -> str:
+        assert self.maestro is not None
         if self.maestro.colors:
             return f'Running target <{color}>{self.name}</{color}>...'
         else:
             return f'Running target {self.name}...'
 
     def standardLogEntry(self, color) -> str:
+        assert self.maestro is not None
         padded_label = self.get_label().ljust(self.maestro.get_max_label_length())
         if self.maestro.colors:
             return f'<{color}>{padded_label}</{color}> {self.get_displayed_name()}'
@@ -160,8 +176,9 @@ class BuildTarget(object):
             return f'{padded_label} {self.get_displayed_name()}'
 
     def logStart(self) -> IndentLogger:
+        assert self.maestro is not None
         color = self.BT_COLOR or 'cyan'
-        pct = round((len(self.maestro.targetsCompleted)/len(self.maestro.targets))*100)
+        pct = round((len(self.maestro.targetsCompleted) / len(self.maestro.targets)) * 100)
         msg = ''
         if self.maestro.verbose:
             msg = self.verboseLogEntry(color)
@@ -180,26 +197,26 @@ class BuildTarget(object):
         }
 
     def getFilesToCompare(self) -> List[str]:
-        return [os.path.abspath(__file__)]+callLambda(self.files)+self.provides()+self.dependencies
+        return [os.path.abspath(__file__)] + callLambda(self.files) + self.provides() + self.dependencies
 
     def serialize_file_times(self) -> Dict[str, float]:
-        file_mtimes={}
+        file_mtimes = {}
         for filename in self.getFilesToCompare():
             if os.path.isfile(filename):
-                file_mtimes[os.path.abspath(filename)]=os.path.getmtime(filename)
+                file_mtimes[os.path.abspath(filename)] = os.path.getmtime(filename)
         return file_mtimes
 
     def serialize_file_hashes(self) -> Dict[str, str]:
-        file_hashes={}
+        file_hashes = {}
         for filename in self.getFilesToCompare():
             if os.path.isfile(filename):
-                file_hashes[os.path.abspath(filename)]=utils.hashfile(filename, hashlib.md5())
+                file_hashes[os.path.abspath(filename)] = utils.hashfile(filename, hashlib.md5())
         return file_hashes
 
-    def genVirtualTarget(self, vid: Optional[str] = None) -> str:
+    def genVirtualTarget(self, bm: BuildMaestro, vid: Optional[str] = None) -> str:
         if vid is None:
             vid = self.getConfigHash()
-        return os.path.join(self.maestro.builddir, 'tmp', 'virtual-targets', vid)
+        return os.path.join(bm.builddir, 'tmp', 'virtual-targets', vid)
 
     def deserialize(self, data: Dict[str, Any]) -> None:
         self.target = data['target']
@@ -208,7 +225,8 @@ class BuildTarget(object):
         self._all_provides = data.get('provides', [])
 
     def getCacheFile(self) -> str:
-        filename = hashlib.md5(self.name.encode('utf-8')).hexdigest()+'.yml'
+        assert self.maestro is not None
+        filename = hashlib.md5(self.name.encode('utf-8')).hexdigest() + '.yml'
         return os.path.join(self.maestro.builddir, 'cache', filename)
 
     def getConfigHash(self) -> str:
@@ -227,22 +245,22 @@ class BuildTarget(object):
             yaml.dump_all([self.CACHE_VER, configHash, targetHash, self.serialize_file_times(), self.serialize_file_hashes(), self.get_config()], f)
 
     def readCache(self) -> None:
-        self.lastConfigHash=''
-        self.lastTargetHash=''
-        self.lastFileTimes={}
-        self.lastFileHashes={}
-        self.lastConfig={}
+        self.lastConfigHash = ''
+        self.lastTargetHash = ''
+        self.lastFileTimes = {}
+        self.lastFileHashes = {}
+        self.lastConfig = {}
         if os.path.isfile(self.getCacheFile()):
             try:
                 with open(self.getCacheFile(), 'r') as f:
                     cachedata = list(yaml.load_all(f))
-                    if len(cachedata)==6 and cachedata[0] == self.CACHE_VER:
+                    if len(cachedata) == 6 and cachedata[0] == self.CACHE_VER:
                         _, _CH, _TH, _LFT, _LFH, _CFG = cachedata
-                        self.lastConfigHash=_CH
-                        self.lastTargetHash=_TH
-                        self.lastFileTimes=_LFT
-                        self.lastFileHashes=_LFH
-                        self.lastConfig=_CFG
+                        self.lastConfigHash = _CH
+                        self.lastTargetHash = _TH
+                        self.lastFileTimes = _LFT
+                        self.lastFileHashes = _LFH
+                        self.lastConfig = _CFG
             except Exception as e:
                 log.exception(e)
                 pass
@@ -253,7 +271,7 @@ class BuildTarget(object):
         curFileTimes = self.serialize_file_times()
         if self.CHECK_MTIMES:
             for filename, mtime in self.lastFileTimes.items():
-                filename=os.path.abspath(filename)
+                filename = os.path.abspath(filename)
                 if filename not in curFileTimes.keys():
                     log.debug('File %s is currently missing.', filename)
                     return True
@@ -263,7 +281,7 @@ class BuildTarget(object):
         if self.CHECK_HASHES:
             curFileHashes = self.serialize_file_hashes()
             for filename, hashed in self.lastFileHashes.items():
-                filename=os.path.abspath(filename)
+                filename = os.path.abspath(filename)
                 if filename not in curFileHashes.keys():
                     log.debug('File %s is currently missing.', filename)
                     return True
@@ -271,7 +289,7 @@ class BuildTarget(object):
                     log.debug('File %s has a changed hash. (%s != %s)', filename, curFileHashes[filename], hashed)
                     return True
         for filename, mtime in curFileTimes.items():
-            filename=os.path.abspath(filename)
+            filename = os.path.abspath(filename)
             if filename in self.maestro.targetsDirty:
                 log.debug('File %s was dirtied by another BuildTarget.', filename)
                 return True
@@ -291,7 +309,7 @@ class BuildTarget(object):
         if self.CHECK_MTIMES:
             curFileTimes = self.serialize_file_times()
             for filename, mtime in self.lastFileTimes.items():
-                filename=os.path.abspath(filename)
+                filename = os.path.abspath(filename)
                 if filename not in curFileTimes.keys():
                     log.debug('File %s is currently missing.', filename)
                     o.add(filename)
@@ -301,7 +319,7 @@ class BuildTarget(object):
         if self.CHECK_HASHES:
             curFileHashes = self.serialize_file_hashes()
             for filename, hashed in self.lastFileHashes.items():
-                filename=os.path.abspath(filename)
+                filename = os.path.abspath(filename)
                 if filename not in curFileHashes.keys():
                     log.debug('File %s is currently missing.', filename)
                     o.add(filename)
@@ -309,15 +327,14 @@ class BuildTarget(object):
                     log.debug('File %s has a changed hash.', filename)
                     o.add(filename)
         for filename, mtime in curFileTimes.items():
-            filename=os.path.abspath(filename)
+            filename = os.path.abspath(filename)
             if filename not in self.lastFileTimes.keys():
                 log.debug('File %s is new.', filename)
                 o.add(filename)
         return o
 
-
-    def checkMTimes(self, inputs: List[Union[str, Callable[[], str]]], targets: List[str], config: Optional[Dict[str, Any]]=None) -> bool:
-        inputs=callLambda(inputs)
+    def checkMTimes(self, inputs: List[Union[str, Callable[[], str]]], targets: List[str], config: Optional[Dict[str, Any]] = None) -> bool:
+        inputs = callLambda(inputs)
         for target in targets:
             if not os.path.isfile(target):
                 log.debug('%s does not exist.', target)
@@ -375,21 +392,21 @@ class BuildTarget(object):
 
     def canBuild(self, maestro: 'BuildMaestro', keys: List[str]) -> bool:
         #self.files = list(callLambda(self.files))
-        #for dep in list(set(self.dependencies + self.files)):
+        # for dep in list(set(self.dependencies + self.files)):
         if not self._lambdas_called:
             for reqfile in callLambda(self.files):
                 if reqfile in keys and reqfile not in self.dependencies:
                     self.dependencies.append(reqfile)
         for dep in list(set(self.dependencies)):
             if dep not in maestro.targetsCompleted:
-                log.debug('%s: Waiting on %s.',self.name,dep)
+                log.debug('%s: Waiting on %s.', self.name, dep)
                 return False
-        log.debug('%s: CAN BUILD!',self.name)
+        log.debug('%s: CAN BUILD!', self.name)
         return True
 
     def touch(self, filename: str) -> None:
         p = Path(filename)
-        p.parent.mkdir(exist_ok=True)
+        p.parent.mkdir(exist_ok=True, parents=True)
         p.touch(exist_ok=True)
 
     def _dumpStacks(self) -> None:
@@ -402,5 +419,5 @@ class BuildTarget(object):
 
 class SingleBuildTarget(BuildTarget):
     def __init__(self, target: str, files: List[str] = [], dependencies: List[str] = [], provides: List[str] = [], name: str = '') -> None:
-        self.target=target
+        self.target = target
         super(SingleBuildTarget, self).__init__([target], files=files, dependencies=dependencies, provides=provides, name=name)
